/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import java.io.IOException;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.util.TraversingItemVisitor;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.JspFragment;

import com.squeakysand.jcr.SimpleNode;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@JspTag(
    name = "traverse",
    bodyContent = BodyContent.SCRIPTLESS
)
public class RepositoryTraversingTag extends JcrSessionAwareSimpleTagSupport {

    @JspTagVariable(
        variableClass = SimpleNode.class
    )
    private static final String CURRENT_NODE_VARIABLE = AttributeNames.CURRENT_NODE;

    @JspTagVariable(
        variableClass = Property.class
    )
    private static final String CURRENT_PROPERTY_VARIABLE = AttributeNames.CURRENT_PROPERTY;

    @JspTagVariable(
        variableClass = Integer.class
    )
    private static final String CURRENT_LEVEL_VARIABLE = "level";

    private static final Logger LOG = LoggerFactory.getLogger(RepositoryTraversingTag.class);

    private JspFragment endNodeFragment;
    private JspFragment endPropertyFragment;
    private Node startNode;
    private JspFragment startNodeFragment;
    private JspFragment startPropertyFragment;

    @Override
    public void doTag() throws JspException, IOException {
        TraversingItemVisitor visitor = new TraversingItemVisitor() {

            @Override
            protected void entering(Property property, int level) throws RepositoryException {
                getJspContext().setAttribute(CURRENT_PROPERTY_VARIABLE, property);
                getJspContext().setAttribute(CURRENT_LEVEL_VARIABLE, level);
                try {
                    getStartPropertyFragment().invoke(null);
                } catch (JspException e) {
                    throw new RepositoryException(e);
                } catch (IOException e) {
                    throw new RepositoryException(e);
                }
            }

            @Override
            protected void entering(Node node, int level) throws RepositoryException {
                getJspContext().setAttribute(CURRENT_NODE_VARIABLE, node);
                getJspContext().setAttribute(CURRENT_LEVEL_VARIABLE, level);
                try {
                    getStartNodeFragment().invoke(null);
                } catch (JspException e) {
                    throw new RepositoryException(e);
                } catch (IOException e) {
                    throw new RepositoryException(e);
                }
            }

            @Override
            protected void leaving(Property property, int level) throws RepositoryException {
                getJspContext().setAttribute(CURRENT_PROPERTY_VARIABLE, property);
                getJspContext().setAttribute(CURRENT_LEVEL_VARIABLE, level);
                try {
                    getEndPropertyFragment().invoke(null);
                } catch (JspException e) {
                    throw new RepositoryException(e);
                } catch (IOException e) {
                    throw new RepositoryException(e);
                }
            }

            @Override
            protected void leaving(Node node, int level) throws RepositoryException {
                getJspContext().setAttribute(CURRENT_NODE_VARIABLE, node);
                getJspContext().setAttribute(CURRENT_LEVEL_VARIABLE, level);
                try {
                    getEndNodeFragment().invoke(null);
                } catch (JspException e) {
                    throw new RepositoryException(e);
                } catch (IOException e) {
                    throw new RepositoryException(e);
                }
            }

        };
        try {
            determineTargetNode().accept(visitor);
        } catch (RepositoryException e) {
            throw new JspException(e);
        }
    }

    public Node getStartNode() {
        return startNode;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setStartNode(Node startNode) {
        this.startNode = startNode;
    }

    public JspFragment getEndNodeFragment() {
        return endNodeFragment;
    }

    @JspTagAttribute
    public void setEndNodeFragment(JspFragment endNodeFragment) {
        this.endNodeFragment = endNodeFragment;
    }

    public JspFragment getEndPropertyFragment() {
        return endPropertyFragment;
    }

    @JspTagAttribute
    public void setEndPropertyFragment(JspFragment endPropertyFragment) {
        this.endPropertyFragment = endPropertyFragment;
    }

    public JspFragment getStartNodeFragment() {
        return startNodeFragment;
    }

    @JspTagAttribute
    public void setStartNodeFragment(JspFragment startNodeFragment) {
        this.startNodeFragment = startNodeFragment;
    }

    public JspFragment getStartPropertyFragment() {
        return startPropertyFragment;
    }

    @JspTagAttribute
    public void setStartPropertyFragment(JspFragment startPropertyFragment) {
        this.startPropertyFragment = startPropertyFragment;
    }

    private SimpleNode determineTargetNode() {
        SimpleNode result = null;
        Node targetNode = null;
        // first check if a node was passed via an attribute
        if (startNode != null) {
            LOG.debug("node was passed in via an attribute");
            targetNode = startNode;
        } else {
            // if no node was passed via an attribute, then see if there is a current node in the pageContext
            try {
                targetNode = (Node) getJspContext().findAttribute(AttributeNames.CURRENT_NODE);
                LOG.debug("node was found via current node attribute");
            } catch (ClassCastException e) {
                LOG.error(e.getMessage(), e);
            }

        }
        LOG.debug("target node was " + targetNode);
        if (targetNode != null) {
            if (targetNode instanceof SimpleNode) {
                result = (SimpleNode) targetNode;
            } else {
                result = new SimpleNode(targetNode);
            }
        }
        LOG.debug("result node was " + result);
        return result;
    }
}
